<?php
/********************************************************************
Product		: Flexicontact
Date		: 25 July 2017
Copyright	: Les Arbres Design 2010-2017
Contact		: http://www.lesarbresdesign.info
Licence		: GNU General Public License
*********************************************************************/
defined('_JEXEC') or die('Restricted Access');

class Flexi_captcha
{

// -------------------------------------------------------------------------------
// Display the image captcha
// Builds a structure containing information about the test and stores it in the session
// Returns the description of the target image
//
static function show_image_captcha($config_data, $error)
{
// get list of images in images directory

    $image_info = Flexicontact_Utility::get_captcha_image_path();
    $image_path = $image_info['path'];
    $image_url = $image_info['url'];
    FC_trace::trace("Image path is $image_path");
    $handle = opendir($image_path);
	if (!$handle)
        {
        FC_trace::trace("No images in $image_path");
		return "Images directory not found";      // should never happen
        }
		
    $image_list = array();
	while (($filename = readdir($handle)) != false)
	    {
    	if ($filename == '.' or $filename == '..')
    		continue;
    	$imageInfo = @getimagesize($image_path.'/'.$filename);
    	if ($imageInfo === false)
    		continue;				            // not an image
    	if ($imageInfo[3] > IMAGETYPE_PNG)      // only support gif, jpg or png
    		continue;
    	if ($imageInfo[0] > 150)                // if X size > 150 pixels ..
    		continue;                           // .. it's too big so skip it
    	$image = array();
    	$image['filename'] = $filename;
    	$image['width'] = $imageInfo[0];
    	$image['height'] = $imageInfo[1];
    	$image['type'] = $imageInfo[2];
    	$image_list[] = $image;
    	}
    closedir($handle);
    
    if (empty($image_list))
        {
        FC_trace::trace("No suitable images in $image_path");
		return "No suitable images in images directory";
        }
		
	$imageCount = count($image_list);
	if ($imageCount < $config_data->num_images)
        {
        FC_trace::trace("Only $imageCount images in $image_path but requested ".$config_data->num_images);
		return 'Not enough images in images directory. Requested: '.$config_data->num_images.' Found: '.$imageCount.'<br />';
        }

// load the Javascript

	$js= "function imageSelect(pictureID) {var images = document.getElementsByTagName('img');
  for (var i = 0; i < images.length; i++) if (images[i].className == 'fc_active') images[i].className = 'fc_inactive';
  document.getElementById(pictureID).className = 'fc_active'; document.getElementById('fc_picselected').value = pictureID; }\n";
	$doc = JFactory::getDocument();
	$doc->addScriptDeclaration($js);

// choose the images
	
	$i = 0;
	$randoms = array();
	while ($i < $config_data->num_images)
		{
		$imageNum = rand(0,$imageCount - 1);	// get a random number
		if (in_array($imageNum,$randoms))		// if already chosen
			continue;							// try again
		$randoms[] = $imageNum;					// add to random number array
		$i ++;									// got one more
		}

// build the captcha information structure

	$captcha_info = new stdClass();
	$captcha_info->num_images = $config_data->num_images;
	$captcha_info->images = array();

	for ($i = 0; $i < $config_data->num_images; $i++)
		{
		$j = $randoms[$i];						// index of the next chosen image
		$image = $image_list[$j];				// point to image info
		$captcha_info->images[$i] = $image;		// copy the image info array into the captcha_info structure
		}
		
// choose the target image and store it in the captcha_info structure
	
	$captcha_info->target = rand(0, $config_data->num_images - 1);
	$target_filename = $captcha_info->images[$captcha_info->target]['filename'];
	FC_trace::trace("Created captcha_info structure: ".print_r($captcha_info,true));	

// store the captcha_info structure in the session, and check it got stored correctly

	$app = JFactory::getApplication();
	$app->setUserState(LAFC_COMPONENT."_captcha_info", $captcha_info);
    $captcha_info_check = $app->getUserState(LAFC_COMPONENT."_captcha_info",'');
    if ($captcha_info_check == '')
        {
    	FC_trace::trace("Captcha info session check failed");	
        return 'Cannot use image captcha because the Joomla session is not working correctly';
        }
	$retry_count = $app->getUserState(LAFC_COMPONENT."_captcha_retry_count",0);

// load the additional language file provided by our image packs

	$lang = JFactory::getLanguage();
	$lang->load('com_flexicontact_captcha', JPATH_SITE);

// load any additional language files provided by new xml image packs

	$language = $lang->get('tag');
    $lang_files = glob(JPATH_SITE.'/components/com_flexicontact/language/'.$language.'/'.$language.'.com_flexicontact_theme_*.ini');
    foreach ($lang_files as $lang_file)
        {
        FC_trace::trace("Loading language file: $lang_file");
        $base_name = basename($lang_file);
        $file_name = substr($base_name,6,(strlen($base_name)-10));  // strip off the language and the .ini
    	$lang->load($file_name, JPATH_SITE.'/components/com_flexicontact');
        }
        
// get the description of the target image and make the user prompt
	
	$target_text = JText::_('COM_FLEXICONTACT_IMAGE_'.strtoupper($target_filename));
	$select_image_text = JText::_('COM_FLEXICONTACT_SELECT_IMAGE');
	if (strstr($select_image_text,"%s"))
        $text = JText::sprintf('COM_FLEXICONTACT_SELECT_IMAGE',$target_text);
	else
		$text = JText::_('COM_FLEXICONTACT_SELECT_IMAGE').' '.$target_text;
//    $text .= ' ('.$retry_count.')';   // useful for testing

// draw the chosen images
        
	$html = "\n".'<div class="fc_line fc_images">';
	$html .= "\n".'<label class="fc_image_text">';
	$html .= '<span class="fc_image_desc">'.$text.'</span>';
	$html .= "\n".'</label>';
	$html .= '<div class="fc_image_inner">';
	foreach ($captcha_info->images as $index => $image)
		{
		$img_id = sprintf('fc_image_%03d', $index);
		$img_name = $image['filename'];
		$image_height = $image['height'];
		$image_width = $image['width'];
			
		if ($config_data->raw_images)
			$src = $image_info['url'].$img_name;
		else
			$src = JURI::root(true).'/index.php?option='.LAFC_COMPONENT.'&amp;tmpl=component&amp;format=raw&amp;lang=en&amp;task=image&amp;n='.$index.'&amp;r='.mt_rand();
            
		$imageHtml = '<img id="i_'.$img_id.'" src="'.$src.'" width="'.$image_width.'" height="'.$image_height.'" 
			alt="" class="fc_inactive" onclick="imageSelect('."'i_".$img_id."'".')" />';

		$html .= "\n".$imageHtml;
		}
	$html .= "\n".'</div>';
    
// do we have an error message?

	if ($error != '')
		$html .= "\n".$error;
		
	$html .= "\n".'<input type="hidden" name="picselected" id="fc_picselected" value="" />';
	$html .= "\n".'</div>';     // class="fc_line fc_images"
	return $html;
}

// -------------------------------------------------------------------------------
// Serve an image to the browser
//
static function show_image()
{
	$app = JFactory::getApplication();
	$captcha_info = $app->getUserState(LAFC_COMPONENT."_captcha_info",'');
	if ($captcha_info == NULL)
        {
       	FC_trace::trace("Unable to serve image because there is no session data");
        header('Warning: Session data not found');
        http_response_code(403);    // Unauthorised
		return;
        }        
	
	$jinput = JFactory::getApplication()->input;
	$image_number = $jinput->get('n', 0, 'INT');
	$filename = $captcha_info->images[$image_number]['filename'];
    $image_info = Flexicontact_Utility::get_captcha_image_path();
	$filepath = $image_info['path'].'/'.$filename;
   	FC_trace::trace("Serving image $image_number = $filepath");	

// set the mime type

	switch ($captcha_info->images[$image_number]['type']) 
		{
		case 1:  
			$mimetype = 'image/gif';
			break;
		case 2:
			$mimetype = 'image/jpeg';
			break;
		case 3:
			$mimetype = 'image/png';
			break;
		default: return;
		}
		
	while (@ob_end_clean());
	header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
	header("Last-Modified: ".gmdate("D, d M Y H:i:s") . "GMT");
	header("Cache-Control: no-store, no-cache, must-revalidate");
	header("Cache-Control: post-check=0, pre-check=0", false);
	header("Pragma: no-cache");
	header("Content-Type: $mimetype");
	echo readfile($filepath);
	echo self::random_data();
	exit;
}

// -------------------------------------------------------------------------------
// Make some random data to vary the image file length
//
static function random_data()
{
	$str = '';
	$length = rand(0,200);
	for ($i = 0; $i < $length; $i++)
		$str .= chr(rand(0,255));
	return $str;
}

// -------------------------------------------------------------------------------
// Check if the user picked the correct image
// returns 0 for yes, the user picked the correct image
//         1 for no, the user picked the wrong image
//         2 if the user picked the wrong image more than (num_images - 1) times
//
static function check($pic_selected, $num_images)
{
	$app = JFactory::getApplication();
	$captcha_info = $app->getUserState(LAFC_COMPONENT."_captcha_info",'');
	if ($captcha_info == '')
        {
    	FC_trace::trace("Flexi_captcha::check() no session data");
		return 1;
        }
    FC_trace::trace("Flexi_captcha::check() captcha_info: ".print_r($captcha_info,true));
	$target_img_id = sprintf('fc_image_%03d', $captcha_info->target);
	FC_trace::trace("Flexi_captcha::check() pic_requested = $target_img_id, pic_selected = $pic_selected");

	if (($pic_selected == '') or (strcmp($target_img_id,$pic_selected) != 0))
		{
		$retry_count = $app->getUserState(LAFC_COMPONENT."_captcha_retry_count",0);
		$retry_count ++;
		$app->setUserState(LAFC_COMPONENT."_captcha_retry_count", $retry_count);
       	FC_trace::trace("Flexi_captcha::check() failed, retry count is $retry_count, retry limit is ".($num_images - 1));
		if ($retry_count <= ($num_images - 1))
			return 1;
		else
			return 2;
		}
   	FC_trace::trace("Flexi_captcha::check() OK");
	$app->setUserState(LAFC_COMPONENT."_captcha_info", '');	// destroy the session captcha info
	return 0;
}

}


